/*--------------------------------------------------------------------------
 * Copyright (C) 1992,1993,1994,1995,1996,1997 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  TMEPROM.C - This utility uses TMEX to view and program iButton Eproms.  It
 *              is currently limited to the DS1982.
 *
 *  Compiler:  Microsoft Visual C 
 *  Version: 3.00Alpha
 *
 */

#define TMEXUTIL

#include "iBTMEX.H"   

/* Local Function Prototypes */
short FindFirstFamily(short, long);
void Exit_Prog(char *, short);
ushort docrc16(ushort);
short DumpDevice(void);
short ToHex(char);
short EnterHex(char *, short, ushort *);
short ProgramByte(ushort, ushort, ushort);
short iButtonFound(void);

/* Global Variables to hold list information */
short PortType, PortNum;
uchar state_buffer[5120];
ushort CRC16;
short ROM[9];

/*----------------------------------------------------------------------
 *  This is the Main routine for SWVC32.
 */
void main(short argc, char **argv)
{
   short done,ClearActivity=0;
   char tstr[80],ch;       
   char ReturnString[256];
   HKEY GlobKey;
   DWORD dwType,size;
   ushort pagetype,address,value;

   /* check arguments to see if request instruction with '?' or too many */
   if ((argc > 2) || ((argc == 2) && (argv[1][0] == '?' || argv[1][1] == '?')))
   {
       printf("\nusage: tmeprom <PortNum> \n"
              "  - program individual bytes\n"
              "  - <optional> argument 1 specifies the port number\n"
              "  - version 3.00Alpha\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   /* read the default PortNum and PortType from the registry */
   if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
         "Software\\Dallas Semiconductor\\iButton TMEX\\3.00",0,
         KEY_READ,&GlobKey) == ERROR_SUCCESS)
   {
      /* attempt to read the PortNum */
      size = 255;
      PortNum = 1;
      if (RegQueryValueEx(GlobKey,"DefaultPortNum",NULL,&dwType,
          &ReturnString[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) || (size > 0))
            PortNum = atoi(ReturnString);
      }
      /* attempt to read the PortType */
      size = 255;
      PortType = 1;
      if (RegQueryValueEx(GlobKey,"DefaultPortType",NULL,&dwType,
          &ReturnString[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) || (size > 0))
            PortType = atoi(ReturnString);
      }

      // close the key
      RegCloseKey(GlobKey);
   }
   else
     Exit_Prog("ERROR, Could not read the default PortNum and PortType from registry!\n",
               NO_DRIVERS);

   /* check argument to see if provided port number */
   if (argc > 1)
       PortNum = atoi(argv[1]);

   /* check to see that indicated PortNum is  <= 15 */
   if (PortNum > 15)
     Exit_Prog("ERROR, Indicated PortNum is not valid!\n",
               -ONE_WIRE_PORT_ERROR);

   /* print a header */  
   printf("\nTMEPROM Version 3.00Alpha\n"
         "Dallas Semiconductor Corporation\n"
         "Copyright (C) 1992-1997\n\n");
   printf("Port number: %d    Port type: %d\n",PortNum,PortType);
   Get_Version(tstr);
   printf("Main Driver: %s\n",tstr);
   printf("Type%d:",PortType);
   if (TMGetTypeVersion(PortType,tstr) < 0)
      Exit_Prog("No Hardware Driver for this type found!",NO_DRIVERS);
   printf(" %s\n\n\n",tstr);


   /* continue if an iButton found */
   if (iButtonFound())                          
   {   
      /* always display info first */
      DumpDevice();

      /* loop while not done */
      done = FALSE;
      do
      {  
         /* print menu */
         printf("\n\n(1) Display the memory contents\n"
              "(2) Program a byte\n"
              "(3) Find a new DS1982\n"
              "(4) Quit\n\n"
              "Select a Number:");
         ch = getche();
         printf("\n\n");
         
         /* do something from the menu selection */             
         ClearActivity = FALSE;
         switch(ch)
         {        
            case '1': 
               DumpDevice();
               break;
            case '2':
               if (EnterHex("Enter the page type (0 regular, 1 status): ",1,&pagetype))
                  if (EnterHex("Enter the 4 character hex address: ",4,&address))
                     if (EnterHex("Enter the 2 character hex value: ",2,&value))
                        ProgramByte(address,value,pagetype);
               DumpDevice();
               break;
            case '3':
               if (!iButtonFound())
                  done = TRUE;
               else
                  DumpDevice();
               break;
            case '4': case 'q': case 'Q':
               done = TRUE;  
               break;                             

            default:
               MessageBeep(MB_ICONEXCLAMATION);
         };
      }
      while (!done);
   }
}


/*----------------------------------------------------------------------
 * Enter a hex string and convert it to an unsigned short
 */                                                          
short iButtonFound(void)
{
   short printmsg=1,found;
   long session_handle;                         

   /* loop to get a session handle find a switch */
   found = FALSE;
   for (;;)
   {
      /* attempt to get a session */
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         /* setup the MicroLan */
         if (TMSetup(session_handle) == 1)
         {
            printf("Searching for a DS1982...\n\n");
            
            /* look for the first DS1982 iButton (family type 0x09) */
            if (FindFirstFamily(0x09,session_handle))
            {
               found = TRUE;
               break;
            }
            /* Temperature iButton not on MicroLan */
            else   
            {
               printf("DS1982 not found on MicroLan!"); 
               break;
            }
         }   
         /* MicroLan port not valid */
         else   
         {
            printf("MicroLan setup failed!"); 
            break;
         }
      }
      else if (session_handle == 0)
      {                            
         /* check if need to print the waiting message */
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the MicroLan\n\n");
         }
      }
      else if (session_handle < 0)   
      {
         MessageBeep(MB_ICONEXCLAMATION);
         printf("No Hardware Driver for this type found!");
         break;
      }
   }

   /* end the session if one is open */
   TMEndSession(session_handle);

   return found;
}


/*----------------------------------------------------------------------
 */                                                          
short ProgramByte(ushort address, ushort value, ushort pagetype)
{
   short printmsg=1,pg,flag,rt=0,cnt=0,done=0,bits;
   long session_handle;    
   uchar pgbuf[50];                     

   printf("\nProgramming byte %04X to the value %02X\n",address,value);


   // loop to get a session handle and to a temperature conversion */
   do
   {
      /* attempt to get a session */
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {

         bits = 8;
         flag = TMProgramByte(session_handle,&state_buffer,value,address,pagetype,&bits,1);
         printf("Result of program byte %d with a final number bits %d\n",flag,bits);
         
         // read back the byte to check
         pg = (short)((address / 32) + ((address % 32) ? 1 : 0) - 1);
         if (pg < 0)
            pg = 0;
         flag = TMExtendedReadPage(session_handle,&state_buffer,
                                   pg,pgbuf,pagetype);
         if (flag < 0)
         {
           printf("ERROR trying to read result of program.\n");
           MessageBeep(MB_ICONEXCLAMATION);
         }
         else
         {
            printf("Byte read back is %02X\n",pgbuf[address % 32]);
            if (~value & pgbuf[address % 32])
            {
               printf("ERROR byte not written, check hardware!\n"); 
               MessageBeep(MB_ICONEXCLAMATION);
            }
         }
         printf("\n");

         done = 1;

         /* end the session if one is open */
         TMEndSession(session_handle);
      }
      else if (session_handle == 0)
      {                            
         /* check if need to print the waiting message */
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the MicroLan\n\n");
         }
      }
   }
   while (!done && (cnt++ < 10));

   return done;                  
}


/*----------------------------------------------------------------------
 * Enter a hex string and convert it to an unsigned short
 */                                                          
short EnterHex(char *msg, short numchars, ushort *value)
{
   short tmp,cnt;
   char ch;

   printf("%s",msg);

   *value = 0;

   cnt = 0;
   do
   {
      ch = getche();

      if (ch == 0x08)
      {
         printf(" %c",ch);
         if (cnt)
            cnt--;
      }
      else if (ch == 0x1B)
      {
         printf("  Aborted\n\n");
         return 0;
      }
      else
      {
         tmp = ToHex(ch);
         if (!tmp && (ch != '0'))
         {
            printf("[%02X]",ch);
            MessageBeep(MB_ICONEXCLAMATION);
         }
         else
         {
            *value <<= 4; 
            *value |= tmp;
            cnt++;
         }   
      }
   }
   while (cnt < numchars);

   printf("\n");

   return 1;
}


/*----------------------------------------------------------------------
 */                                                          
short DumpDevice(void)
{    
   short i,j,k,printmsg=1,flag,rt=0,cnt=0,done=0;
   long session_handle;    
   uchar pgbuf[50];                     

   // loop to get a session handle and to a temperature conversion */
   do
   {
      /* attempt to get a session */
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         // assume it will work
         done = 1;

         printf("Regular Memory:\n");
         
         /* loop to read all of the regular pages */
         for (i = 0; i <= FamSpec[ROM[0] & 0x7F].maxp; i++)
         {
            /* read the page */
            flag = TMExtendedReadPage(session_handle,&state_buffer,i,pgbuf,0);

            if (flag > 0)
            {
                for (k = 0; k < 2; k++)
                {
                     printf("%04X  ",k*16 + i*32);

                     for (j = 0; j < 16; j++)
                         printf("%02X ",pgbuf[k*16 + j]);
                     printf("\n");
                }
            }
            else
            {
               done = 0;
               break;
            }
         }
                                
         printf("Status Memory:\n");
         
         /* read the status pages */
         flag = TMExtendedReadPage(session_handle,&state_buffer,0,pgbuf,1);

         if (flag > 0)
         {
            printf("%04X  ",0);

            for (j = 0; j < flag; j++)
                printf("%02X ",pgbuf[j]);
            printf("\n");
         }
         else
         {
            done = 0;
         }

         /* end the session if one is open */
         TMEndSession(session_handle);
      }
      else if (session_handle == 0)
      {                            
         /* check if need to print the waiting message */
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the MicroLan\n\n");
         }
      }
   }
   while (!done && (cnt++ < 10));

   return done;                  
}


/*----------------------------------------------------------------------
 * Find the first device with a particular family
 */
short FindFirstFamily(short family, long session_handle)
{
   short j;
   
   /* set up to find the first device with family 'family */
   if (TMFamilySearchSetup(session_handle,&state_buffer,family) == 1)
   { 
      /* get first device in list with the specified family */
      if (TMNext(session_handle,state_buffer) == 1)
      {  
         /* read the rom number */
         ROM[0] = 0;
         TMRom(session_handle,state_buffer,ROM);   
         
         /* check if correct type */
         if ((family & 0x7F) == (ROM[0] & 0x7F))
         {
            printf("Serial ROM ID: ");  
            for (j = 7; j >= 0; j--)
               printf("%02X",ROM[j]);
            printf("\n\n");
            return 1;
         }        
      }
   }
     
   /* failed to find device of that type */
   return 0;                              
}


/*--------------------------------------------------------------------------
 * Calculate a new CRC16 from the input data shorteger.  Return the current
 * CRC16 and also update the global variable CRC16.
 */
static short oddparity[16] = { 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0 };

ushort docrc16(ushort data)
{
   data = (data ^ (CRC16 & 0xff)) & 0xff;
   CRC16 >>= 8;

   if (oddparity[data & 0xf] ^ oddparity[data >> 4])
     CRC16 ^= 0xc001;

   data <<= 6;
   CRC16   ^= data;
   data <<= 1;
   CRC16   ^= data;

   return CRC16;
}
 
 
/*--------------------------------------------------------------------------
 *  Prints a message, beeps and exits the program.
 */
void Exit_Prog(char *msg, short errcd)
{
   /* print the message left justified */
   printf("\r%s",msg);
   
   /*  error beep if return code is not 0 */
   if (errcd != NORMAL_EXIT)
     MessageBeep(MB_ICONEXCLAMATION);
   
   /*  return error code passed */
   exit(errcd);
}


//------------------------------------------------------------------------
// Convert 1 hex character to binary.
//
short ToHex(char ch)
{
   if ((ch >= '0') && (ch <= '9'))
      return ch - 0x30;
   else if ((ch >= 'A') && (ch <= 'F'))
      return ch - 0x37;
   else if ((ch >= 'a') && (ch <= 'f'))
      return ch - 0x57;
   else
      return 0;
}
                           
